/*
 *   The VNet Class Library. Useful Java components.
 *   Copyright (C) 1999 Michael Gratton.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Author: Michael Gratton - mike@vee.net
 *
 * web:   http://web.vee.net/projects/vnetclasslibrary/
 * email: java@vee.net
 */

package vnet.apps.edit;

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

/**
 * A quick and easy text editor. Run from a command line or create a new
 * instance of the Frame from an existing app.
 *
 * <p>Edit dosen't use Swing but needs a JRE/JDK >= 1.1
 * Java Platform.
 */
public class Edit extends java.awt.Frame
    implements
    java.awt.datatransfer.ClipboardOwner,
    java.awt.event.ActionListener {

    Undo currentUndo;
    MenuItem mFind;
    MenuItem mUndo;
    MenuItem mClose;

    FindDialog findDialog;

    File file;

    static Vector openFrames= new Vector();

    Clipboard clipboard= getToolkit().getSystemClipboard();

    static final String TITLE= "VNet Edit v0.9";

    /**
     * Runs the app. The filename of a text to load file may be specifed
     * as the first argument.
     */
    public static void main(java.lang.String[] args) {
	String filename= (args.length > 0) ? args[0] : null;

	Edit edit= new Edit(filename, false);
	edit.setVisible(true);
    }

    /**
     * Construcs a new Edit window. Will attempt to open the passed filename.
     * @param filaneme a filename to open or null for an empty frame.
     */
    public Edit(String filename) {
	this(filename, true);
    }

   /**
     * Construcs a new Edit window. Will attempt to open the passed filename.
     * @param filaneme a filename to open or null for an empty frame.
     * @param noExit if true, will not kill the runtime when closing.
     */
    public Edit(String filename, boolean noExit) {
	super(TITLE);
	
	TextArea area= new TextArea();
	add(area);
	addWindowListener(new EditWindowAdapter());
	openFrames.addElement(this);
	addMenu(noExit);
	
	if (filename!= null) {
	    file= new File(filename);
	    if (file.exists()) {
		setTitle(TITLE + " - " + file);
		openFile(area);
	    }
	    else
		file= null;
	}

	setSize(new Dimension(450, 300));
    }	

    protected void adjustOpenFrames() {
	int size= openFrames.size();

	if (size== 1)
	    ((Edit)openFrames.elementAt(0)).mClose.setEnabled(false);

	else
	    for (int i= 0; i < size; i++)
		((Edit)openFrames.elementAt(i)).mClose.setEnabled(true);

	System.out.println(openFrames);
    }

    protected void removeFrame() {
	openFrames.removeElement(this);
    }
			       

    public void actionPerformed(ActionEvent e) {
	String cmd= e.getActionCommand();
	TextArea area= (TextArea)getComponent(0);

	// File menu
 
	if (cmd== "New") {
	    setTitle(TITLE + " - Untitled");
	    file= null;
	    area.setText("");
	    undo(null);
	}
	
	else if (cmd== "Open...") {
	    FileDialog fd= new FileDialog(this, "Open");
	    fd.setVisible(true);

	    if (fd.getFile()== null)
		return;

	    file= new File(fd.getDirectory()+ fd.getFile());
	    setTitle(TITLE + " - " + file);
	    openFile(area);
	    undo(null);
	}		

	else if (cmd== "Open in new window...") {
	    FileDialog fd= new FileDialog(this, "Open");
	    fd.setVisible(true);

	    if (fd.getFile()== null)
		return;

	    Edit edit = new Edit(fd.getFile(), false);

	    adjustOpenFrames();

	    edit.setVisible(true);
	}		

	else if (cmd== "Save") {
	    if (file != null) {
		saveFile(area);
		return;
	    }

	    else {
		FileDialog fd= new FileDialog(this,
					      "Save as",
					      FileDialog.SAVE);

		fd.setVisible(true);

		if (fd.getFile()== null)
		    return;

		file= new File(fd.getDirectory()+ fd.getFile());
		setTitle(TITLE + " - " + file);
		saveFile(area);
		undo(null);
	    }
	}		

	else if (cmd== "Save as...") {
	    FileDialog fd= new FileDialog(this, "Save as", FileDialog.SAVE);

	    fd.setVisible(true);

	    if (fd.getFile()== null)
		return;

	    file= new File(fd.getDirectory()+ fd.getFile());
	    setTitle(TITLE + " - " + file);
	    saveFile(area);
	    undo(null);
	}		

	else if (cmd== "Close") {
	    openFrames.removeElement(this);
	    adjustOpenFrames();
	    setVisible(false);
	}		

	else if (cmd== "Exit") {
	    System.exit(0);
	}		

	// Edit menu

	else if (cmd== "Undo") {
	    area.setSelectionStart(currentUndo.strPos);
	    area.setSelectionEnd(currentUndo.endPos);
	    Undo tempUndo= new Undo(area.getSelectedText(),
				    currentUndo.strPos,
				    currentUndo.strPos
				    +currentUndo.string.length());
	    
	    area.replaceRange(currentUndo.string,
				    currentUndo.strPos,
				    currentUndo.endPos);

	    undo(tempUndo);
	}	


	else if (cmd== "Cut") {
	    if (area.getSelectionStart() != area.getSelectionEnd()) {
		String srcData = area.getSelectedText();
		StringSelection contents = new StringSelection(srcData);
		int start= area.getSelectionStart();
		int end= area.getSelectionEnd();
		
		clipboard.setContents(contents, this);
		area.replaceRange(null, start, end);
		
		undo(new Undo(srcData, start, start));
	    }
	}	

	else if (e.getActionCommand() == "Copy") {
	    if (area.getSelectionStart() != area.getSelectionEnd()) {
		String srcData= area.getSelectedText();
		StringSelection contents= new StringSelection(srcData);
		clipboard.setContents(contents, this);
		
		undo(null);
	    }
	}	

	else if (e.getActionCommand() == "Paste") {
	    Transferable content= clipboard.getContents(this);
	    if (content != null) {
		try {
		    String dstData= (String)content.
			getTransferData(DataFlavor.stringFlavor);
		    int startPos= area.getSelectionStart();
		    int endPos= area.getSelectionEnd();

		    undo(new Undo(area.getSelectedText(),
				  startPos,
				  startPos+ dstData.length()));
				
		    area.replaceRange(dstData,startPos,endPos);

		}	
		catch (Exception ex) {
		    System.out.println("Couldn't get contents in format: "+
				       DataFlavor.stringFlavor
				       .getHumanPresentableName());
		}
	    }
	}			

	else if (cmd== "Clear") {
	    int start= area.getSelectionStart();
	    int end= area.getSelectionEnd();
	    
	    area.replaceRange(null, start, end);

	    undo(new Undo(area.getSelectedText(), start, start));
	}	

	
	else if (cmd== "Select all") {
	    area.selectAll();
	}	

	else if (cmd== "Find...") {
	    mFind.setEnabled(false);
	    findDialog= new FindDialog(this);
	    findDialog.addActionListener(this);
	}

	// View menu

	else if (cmd== "Serif")
		area.setFont(new Font("Serif", Font.PLAIN, 12));

	else if (cmd== "Sans-serif")
		area.setFont(new Font("Sans-serif", Font.PLAIN, 12));
				
	else if (cmd== "Monospaced")
		area.setFont(new Font("Monospaced", Font.PLAIN, 12));
				
	else if (cmd== "Dialog")
		area.setFont(new Font("Dialog", Font.PLAIN, 12));


	// find dialog

	else if (cmd== "findFind") {
	    String string= new String(findDialog.textField.getText());
	    String text= new String(area.getText());
	    boolean found= false;
	    boolean wrapped= false;
	    int strLoc, textCurrChar;
	    int strLength= string.length();
	    int textLoc= area.getCaretPosition();
	    int textLength= text.length();
	    int textStartLoc= textLoc;
	    
	    // search only if the string is shorter then the text
	    if (strLength <= textLength) {
		
		// check for case sens, if so drop everything to lowercase
		if (findDialog.caseSensitive) {
		    string.toLowerCase();
		    text.toLowerCase();
		}	
		
		++textLoc; // don't find the same thing we just searched for..
		
		// loop until a match is found
		while (!found) {
		    
		    // unless wrapping, exit when the end of the text
		    // is reached
		    if (strLength > (textLength- textLoc)) {
			if ((findDialog.wrap) & (!wrapped)) {
			    textLoc= 0;
			    wrapped= true;
			}	
			else break;
		    }
		    
		    strLoc= 0;
		    if (string.charAt(strLoc) == text.charAt(textLoc)) {
			found= true;
			++strLoc;
			textCurrChar= textLoc+ 1;
			
			while (strLoc < strLength) {
			    
			    if (string.charAt(strLoc)!=
				text.charAt(textCurrChar)) {
				found= false;
				break;
			    }
			    ++textCurrChar;
			    ++strLoc;
			}
		    }
		    
		    ++textLoc;
		} // end loop
	    } // end if	
	    
	    if (found) {
		area.setSelectionStart(textLoc- 1);
		area.setSelectionEnd(textLoc+ strLength -1);
	    }
	    
	    area.requestFocus();
	    
	}
	
	else if (cmd== "findCancel") {
	    findDialog.setVisible(false);
	    findDialog= null;
	    
	    // enable find in the edit menu		
	    mFind.setEnabled(true);
	}	
	
    }

    protected void addMenu(boolean noExit) {
	
	// The File menu
	Menu mFile= new Menu("File");
	
	MenuShortcut msNew = new MenuShortcut(KeyEvent.VK_N);
	MenuItem mNew= new MenuItem("New", msNew);
	mNew.addActionListener(this);
	mFile.add(mNew);
	
	MenuShortcut msOpen = new MenuShortcut(KeyEvent.VK_O);
	MenuItem mOpen= new MenuItem("Open...", msOpen);
	mOpen.addActionListener(this);
	mFile.add(mOpen);

	MenuItem mOpenWin= new MenuItem("Open in new window...");
	mOpenWin.addActionListener(this);
	mFile.add(mOpenWin);
	
	MenuItem mSeparator1= new MenuItem("-");
	mFile.add(mSeparator1);
	
	MenuShortcut msSave = new MenuShortcut(KeyEvent.VK_S);
	MenuItem mSave= new MenuItem("Save", msSave);
	mSave.addActionListener(this);
	mFile.add(mSave);
	
	MenuItem mSaveAs= new MenuItem("Save as...");
	mSaveAs.addActionListener(this);
	mFile.add(mSaveAs);
	
	MenuItem mSeparator2= new MenuItem("-");
	mFile.add(mSeparator2);
	
	mClose= new MenuItem("Close");
	mClose.addActionListener(this);
	mClose.setEnabled(false);
	mFile.add(mClose);

	if (!noExit) {
	    MenuShortcut msExit = new MenuShortcut(KeyEvent.VK_X);
	    MenuItem mExit= new MenuItem("Exit", msExit);
	    mExit.addActionListener(this);
	    mFile.add(mExit);
	}
	
	// The Edit menu
	Menu mEdit= new Menu("Edit");
	
	mUndo= new MenuItem("Undo");
	mUndo.setEnabled(false);
	mUndo.addActionListener(this);
	mEdit.add(mUndo);
	
	MenuItem mSeparator3= new MenuItem("-");
	mEdit.add(mSeparator3);
	
	MenuItem mCut= new MenuItem("Cut");
	mCut.addActionListener(this);
	mEdit.add(mCut);
	
	MenuItem mCopy= new MenuItem("Copy");
	mCopy.addActionListener(this);
	mEdit.add(mCopy);
	
	MenuItem mPaste= new MenuItem("Paste");
	mPaste.addActionListener(this);
	mEdit.add(mPaste);
	
	MenuItem mSeparator4= new MenuItem("-");
	mEdit.add(mSeparator4);
	
	MenuItem mClear= new MenuItem("Clear");
	mClear.addActionListener(this);
	mEdit.add(mClear);
	
	MenuItem mSeparator5= new MenuItem("-");
	mEdit.add(mSeparator5);
	
	MenuShortcut msSelectAll = new MenuShortcut(KeyEvent.VK_SLASH);
	MenuItem mSelectAll= new MenuItem("Select all", msSelectAll);
	mSelectAll.addActionListener(this);
	mEdit.add(mSelectAll);
	
	MenuShortcut msFind = new MenuShortcut(KeyEvent.VK_F);
	mFind= new MenuItem("Find...", msFind);
	mFind.addActionListener(this);
	mEdit.add(mFind);
	
	// The View menu
	Menu mView= new Menu("View");
	
	Menu mFont= new Menu("Font");
	
	MenuItem mSerif= new MenuItem("Serif");
	mSerif.addActionListener(this);
	mFont.add(mSerif);
	
	MenuItem mSans= new MenuItem("Sans-serif");
	mSans.addActionListener(this);
	mFont.add(mSans);
	
	MenuItem mMono= new MenuItem("Monospaced");
	mMono.addActionListener(this);
	mFont.add(mMono);
	
	MenuItem mDialog= new MenuItem("Dialog");
	mDialog.addActionListener(this);
	mFont.add(mDialog);
	mView.add(mFont);
			
	// Create the menubar itself
	MenuBar menuBar= new MenuBar();
	menuBar.add(mFile);
	menuBar.add(mEdit);
	menuBar.add(mView);
	
	setMenuBar(menuBar);
    }

    public void lostOwnership(Clipboard clipboard, Transferable content) {
    }

    protected void openFile(TextArea area) {
	if (file== null || area== null)
	    return;

	BufferedReader textReader= null;
	try {
	    textReader= new BufferedReader(new FileReader(file));
	}
	catch (IOException e) {
	    System.err.println("IO exception opening file " +file +
			       " for reading"); 
	    return;
	}
	
	StringWriter textWriter= new StringWriter();
	int c= 0;		
	try {	
	    while (true) {
		c= textReader.read();
		if (c == -1) break;
		else textWriter.write(c);	
	    }
	}		
	catch (IOException e) {
	    System.err.println("IO Exception reading file while opening");
	    return;
	}

	area.setText(textWriter.toString());
    }

    protected void saveFile(TextArea area) {
	if (file== null || area== null)
	    return;

	BufferedWriter textWriter= null;
	try {
	    textWriter = new BufferedWriter(new FileWriter(file));
	}
	catch (IOException e) {
	    System.err.println("IO exception opening file " +file
			       +" for saving"); 
	    return;
	}

	BufferedReader textReader=
	    new BufferedReader(new StringReader(area.getText()));
	int c= 0;
	try {	
	    while (true) {
		c= textReader.read();
		if (c == -1) break;
		else textWriter.write(c);
	    }
	}		
	catch (IOException e) {
	    System.err.println("IO Exception writing string to file");
	    return;
	}	

	// Try closing everything
	try {
	    textWriter.flush();
	    textWriter.close();
	}				
	catch (IOException e) {
	    System.err.println("IO exception closing reader or writer "
			       +"while saving");
	    return;
	}	
	
    }
    
    protected void undo (Undo undo) {
	if (undo != null) {
	    currentUndo= undo;
	    mUndo.setEnabled(true);
	}	
	else {
	    currentUndo= null;
	    mUndo.setEnabled(false);
	}	
    }
    
    class EditWindowAdapter extends WindowAdapter {
	public void windowClosing(WindowEvent e) {
	    if (openFrames!= null && openFrames.size() >= 2) {
		removeFrame();
		adjustOpenFrames();
		setVisible(false);
	    }
	    else
		System.exit(0);
	}
    }

    class Undo {
	
	public String string;
	public int strPos, endPos;	
	
	Undo (String data, int str, int end) {
	    string= data;
	    strPos= str;
	    endPos= end;
	}
    }

}

