/*
 *   The VNet Class Library. Useful Java components.
 *   Copyright (C) 1999 Michael Gratton.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Author: Michael Gratton - mike@vee.net
 *
 * web:   http://web.vee.net/projects/vnetclasslibrary/
 * email: java@vee.net
 */

package vnet.apps.edit;

import java.awt.*;
import java.awt.event.*;

/**
 * Quick and dirty find dialog
 * 
 */
public class FindDialog extends Dialog
    implements
    java.awt.event.ActionListener,
    java.awt.event.ItemListener,
    java.awt.event.KeyListener,
    java.awt.event.WindowListener {

	public boolean wrap= false;
	public boolean caseSensitive= false;
	public TextField textField;
	private Button bnFind;
	ActionListener actionListener= null;

    public FindDialog(Frame parent) {
	super(parent, "Find", false);
	buildWindow();
    }

    public void actionPerformed(java.awt.event.ActionEvent e) {
	String cmd= e.getActionCommand();

	// "Find" button pressed
	if (cmd== "Find") {
	    if (actionListener != null) {
		actionListener.actionPerformed
		    (new ActionEvent(this,
				     ActionEvent.ACTION_PERFORMED,
				     "findFind"));
	    }	
	}
	
	// "Cancel" button pressed
	else if (cmd== "Cancel") {
	    if (actionListener != null) {
		actionListener.actionPerformed
		    (new ActionEvent(this,
				     ActionEvent.ACTION_PERFORMED,
				     "findCancel"));
	    }	
	}
    }

    public void addActionListener(ActionListener l) {
	actionListener= AWTEventMulticaster.add(actionListener, l);
    }

    private void buildWindow() {
	Dimension findSize= new Dimension(250, 80);
	
	// set the dialog's layout
	GridBagLayout layout= new GridBagLayout();
	GridBagConstraints gbc= new GridBagConstraints();
	setLayout(layout);
	
	// create the TextField and add it to the dialog
	gbc.fill= GridBagConstraints.HORIZONTAL;
	gbc.gridwidth= GridBagConstraints.REMAINDER;
	gbc.anchor= GridBagConstraints.NORTHWEST;
	textField = new TextField(30);
	textField.addKeyListener(this);
	layout.setConstraints(textField, gbc);
	add(textField);
	
	// add the widgets
	gbc.fill= GridBagConstraints.BOTH;
	Panel lowerArea= new Panel();
	layout.setConstraints(lowerArea, gbc);
	add(lowerArea);
	
	GridBagLayout lowerLayout= new GridBagLayout();
	gbc= new GridBagConstraints();
	lowerArea.setLayout(lowerLayout);
	
	gbc.gridwidth= GridBagConstraints.REMAINDER;
	gbc.anchor= GridBagConstraints.WEST;
	Checkbox cbCase= new Checkbox("Case sensitive", null, false);
	cbCase.addItemListener(this);
	lowerLayout.setConstraints(cbCase, gbc);
	lowerArea.add(cbCase);
	
	gbc.weightx= 1.0;
	
	gbc.anchor= GridBagConstraints.WEST;
	Checkbox cbWrap= new Checkbox("Wrap", null, false);
	cbWrap.addItemListener(this);
	gbc.gridwidth= GridBagConstraints.REMAINDER;
	lowerArea.add(cbWrap);

	bnFind= new Button("Find");	//bnFind is an instance variable
	gbc.fill= GridBagConstraints.HORIZONTAL;
	bnFind.setEnabled(false);
	bnFind.addActionListener(this);
	gbc.gridwidth= GridBagConstraints.REMAINDER;
	lowerArea.add(bnFind);
		
	Button bnCancel= new Button("Cancel");
	bnCancel.addActionListener(this);
	gbc.gridwidth= GridBagConstraints.REMAINDER;
	lowerArea.add(bnCancel);
	
	//	setSize(findSize);
	setVisible(true);

	textField.requestFocus();
    }

    public void itemStateChanged(ItemEvent e) {
	
	// "Wrap" checkbox toggled
	if (e.getItem() == "Wrap") {
	    if (e.getStateChange() == e.SELECTED)
		wrap= true;
	    else wrap= false;
	}
	
	// "Case sensitive" checkbox toggled
	else if (e.getItem() == "Case sensitive") {
	    if (e.getStateChange() == e.SELECTED)
		caseSensitive= true;
	    else caseSensitive= false;
	}
    }

    public void keyPressed(java.awt.event.KeyEvent e) {
    }

    public void keyReleased(java.awt.event.KeyEvent e) {
	
	switch (e.getKeyCode()) {
	case e.VK_ESCAPE:
	    actionListener.actionPerformed
		(new ActionEvent(this,
				 ActionEvent.ACTION_PERFORMED,
				 "findCancel"));
	    break;

	case e.VK_ENTER:
	    if (textField.getText().length() > 0)
		actionListener.actionPerformed
		    (new ActionEvent(this,
				     ActionEvent.ACTION_PERFORMED,
				     "findFind"));
	default:
	    if (textField.getText().length() > 0)
		bnFind.setEnabled(true);
	    else
		bnFind.setEnabled(false);
	}	
    }

    public void keyTyped(java.awt.event.KeyEvent e) {
    }

    public void removeActionListener(ActionListener l) {
	actionListener= AWTEventMulticaster.remove(actionListener, l);
    }

    public void windowActivated(java.awt.event.WindowEvent e) {
	textField.requestFocus();
    }

    public void windowClosed(java.awt.event.WindowEvent e) {
    }

    public void windowClosing(java.awt.event.WindowEvent e) {
    }

    public void windowDeactivated(java.awt.event.WindowEvent e) {
    }

    public void windowDeiconified(java.awt.event.WindowEvent e) {
    }

    public void windowIconified(java.awt.event.WindowEvent e) {
    }

    public void windowOpened(java.awt.event.WindowEvent e) {
    }

}
